﻿
using System;
using System.Collections.Generic;
namespace HeuristicLab.Problems.RoutePlanning.Osm {
  #region Highway Types

  // see: https://wiki.openstreetmap.org/wiki/Key:highway
  public enum HighwayType {
    null_type,
    motorway,
    motorway_link,
    trunk,
    trunk_link,
    primary,
    primary_link,
    secondary,
    secondary_link,
    tertiary,
    tertiary_link,
    living_street,
    pedestrian,
    residential,
    unclassified,
    service,
    track,
    bus_guideway,
    raceway,
    road,
    path,
    footway,
    cycleway,
    bridleway,
    steps
  }

  #endregion

  public class Way : OsmBase {
    private List<Node> nodes;

    public Way() {
      nodes = new List<Node>();
    }

    public Way(long id)
      : base(id) {
      nodes = new List<Node>();
    }

    public List<Node> Nodes {
      get { return nodes; }
    }

    public HighwayType HighwayTag {
      get {
        if (!base.Tags.ContainsKey(TagConstants.HighwayTagKey)) {
          return HighwayType.null_type;
        }
        if (Enum.IsDefined(typeof(HighwayType), base.Tags[TagConstants.HighwayTagKey])) {
          HighwayType ht = (HighwayType)Enum.Parse(
            typeof(HighwayType),
            base.Tags[TagConstants.HighwayTagKey],
            true);
          return ht;
        } else {
          return HighwayType.null_type;
        }
      }
    }

    public bool OneWay {
      get {
        if (base.Tags.ContainsKey(TagConstants.OneWayTag)) {
          string value = base.Tags[TagConstants.OneWayTag];
          return value.Equals(TagConstants.YesValue);
        } else {
          return false;
        }
      }
    }

    public bool CanBeTraversed(Node source, Node target) {
      // check if way can be traveld by a specific vehicle (car, public transport, pedestiran, bicycle)
      // if (!CanBeTraversed(Vehicle)) return false
      if (OneWay) {
        // if vehicle == pedestrian or bicylce -> return true
        int sourceIndex = Nodes.IndexOf(source);
        int targetIndex = Nodes.IndexOf(target);
        // order of the nodes determines the driving direction
        return (sourceIndex < targetIndex);
      }
      return true;
    }

    //public bool CanBeTraversed(Vehicle) {}
  }
}
